import connexion
from swagger_server.models.error_model import ErrorModel
from swagger_server.models.new_people import NewPeople
from swagger_server.models.people import People
from datetime import date, datetime
from typing import List, Dict
from six import iteritems
from ..util import deserialize_date, deserialize_datetime

import json
import logging
from ..error import Error
from swagger_server.models.peoples import Peoples, DuplicateError, NotPeopleError, NotFoundPeopleError
from swagger_server.models.people import InvalidGenderError 

def add_people(body=None):
    """
    Add a new people
    create new people
    :param body: People object that needs to be added
    :type body: dict | bytes

    :rtype: People
    """
    try:
        if connexion.request.is_json:
            body = NewPeople.from_dict(connexion.request.get_json())
        people = Peoples().append(body)
        logging.getLogger('connexion.app').warning("create people %s" % people.id)
        return people
    except DuplicateError as e:
        return e.to_problem()
    except NotPeopleError as e:
        return e.to_problem()
    except InvalidGenderError as e:
        return e.to_problem()
    else:
        return Error(status=500, title='Error System', type='UNKNOW', value=e.__str__()).to_problem()   

def delete_people(peopleId):
    """
    Deletes a people
    delete people by Id
    :param peopleId: People id to delete
    :type peopleId: int

    :rtype: None
    """
    try:
        people = Peoples().getPeopleById(peopleId)
        Peoples().remove(people)
        return json.dumps({'status' : 'ok'}), 200
    except NotFoundPeopleError as e:
        return e.to_problem()
    else:
        return Error(status=500, title='Error System', type='UNKNOW', value=e.__str__()).to_problem()   

def get_people(peopleId):
    """
    Find people by ID
    Returns a people
    :param peopleId: ID of people that needs to be fetched
    :type peopleId: int

    :rtype: People
    """
    try:
        return Peoples().getPeopleById(peopleId)
    except NotFoundPeopleError as e:
        return e.to_problem()
    else:
        return Error(status=500, title='Error System', type='UNKNOW', value=e.__str__()).to_problem()   

def get_peoples(gender='all'):
    """
    list of people
    Returns list of people
    :param gender: filter by gender
    :type gender: str

    :rtype: List[People]
    """
    return [ people for people in Peoples() if (people.gender == gender or 'all' == gender)]


def update_people(peopleId, body):
    """
    Updates a people with form data
    update people by Id
    :param peopleId: ID of people that needs to be updated
    :type peopleId: int
    :param body: People object that needs to be updated
    :type body: dict | bytes

    :rtype: None
    """
    try:
        if connexion.request.is_json:
            body = People.from_dict(connexion.request.get_json())
        if body.id != peopleId:
            return Error(status=405, title='invalid INPUT', type='RG-003', value='peopleId is not compatible with people object').to_problem()
        Peoples().update(body)
        return json.dumps({'status' : 'ok'}), 200
    except NotFoundPeopleError as e:
        return e.to_problem()
    except InvalidGenderError as e:
        return e.to_problem()
    else:
        return Error(status=500, title='Error System', type='UNKNOW', value=e.__str__()).to_problem()   

