import flask
import logging

from .models.error_model import ErrorModel
from connexion.problem import problem

logger = logging.getLogger('connexion.app')

class Error(Exception, ErrorModel):
    
    def __init__(self, status=400, title=None, type=None,
                      instance=None, headers=None, ext=None, key=None):
        Exception.__init__(self)
        ErrorModel.__init__(self, status, type, title, key, instance)
        self._key = key
        self.ext = ext
        self.headers = headers
    
    def __str__(self):
        return self._key

    def to_problem(self):
        logger.error("{url} {type} {error}".format(url=flask.request.url, 
                                                        type=self.type,
                                                        error=self.__str__()))
        return problem(status=self.status, title=self.title, detail=self.__str__(),
                      type=self.type, instance=self.instance, headers=self.headers,
                      ext=self.ext)


