# coding: utf-8
import uuid

from .people import People, PeopleError
from .new_people import NewPeople

class DuplicateError(PeopleError):

    def __init__(self, key=""):
        PeopleError.__init__(self,
            status=400,
            title="Duplicate People",
            type="RG-001",
            key=key)

    def __str__(self):
        return "%s already exist" % self._key

class NotPeopleError(PeopleError):
 
    def __init__(self):
        PeopleError.__init__(self,
            status=405,
            title="People error",
            type="INPUT-003")
   
    def __str__(self):
        return "It is not People or NewPeople object"

class NotFoundPeopleError(PeopleError):

    def __init__(self, key=""):
        PeopleError.__init__(self,
            status=4040,
            title="Not Found People",
            type="RG-002",
            key=key)

    def __str__(self):
        return "%s people not found" % self._key


class Singleton(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Peoples(list, metaclass=Singleton):
    """
    Peoples - manage list of people
    """
    def __init__(self):
        list.__init__(self)
    
    def _get_id(self):
        return uuid.uuid1().int 

    def append(self, el):
        if type(el) is NewPeople:
            el = People.from_dict(dict(id=self._get_id(), **el.to_dict())) 
        if type(el) is People:
            if el in self:
                raise DuplicateError(el)
            list.append(self,el)
            return el
        raise NotPeopleError()
    
    def getPeopleById(self,id):
        for people in self:
            if people.id == id:
                return people
        raise NotFoundPeopleError(id)        

    def update(self, el):
        if type(el) is not People:
            raise NotPeopleError()
        self.remove(self.getPeopleById(el.id))
        self.append(el)

